// nlAutoLogin.java v0.2 written by Yak!

package extensions;

import java.io.IOException;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import java.io.OutputStream;
import java.io.PrintStream;
import java.net.MalformedURLException;
import java.net.URISyntaxException;
import java.net.URI;
import java.net.URLConnection;

import dareka.common.Logger;
import dareka.common.Config;
import dareka.extensions.Extension2;
import dareka.extensions.ExtensionManager;
import dareka.extensions.Rewriter;
import dareka.processor.HttpResponseHeader;

public class nlAutoLogin implements Extension2, Rewriter {
	// Extension2 interface
	public void registerExtensions(ExtensionManager mgr) {
		mgr.registerRewriter(this);
	}
	public String getVersionString() {
		return "nlAutoLogin v0.1";
	}
	
	// Rewriter interface
	public Pattern getRewriterSupportedURLAsPattern()
	{
		return Pattern.compile("http://[^.]+\\.nicovideo\\.jp/watch/.*");
	}
	public String onMatch(Matcher match, HttpResponseHeader responseHeader, String content) throws IOException
	{
		String auth_state = responseHeader.getMessageHeader("x-niconico-authflag");
		if (auth_state == null) {
			Logger.info("nlAutoLogin: x-niconico-authflag is not found, do nothing.");
		} else if(auth_state.equals("0")) {
			Logger.info("nlAutoLogin: Not-authentificated access is detected for " + match.group());

			try {

				// Authentification process
				// To avoid interleaved authentification, using `synchronized'.
				synchronized(this) {
					URI uri = new URI("https://secure.nicovideo.jp/secure/login?site=niconico");
					URLConnection connection = uri.toURL().openConnection();

					connection.setDoOutput(true);
					OutputStream os = connection.getOutputStream();
					String mail = Config.getString("mail", "");
					String password = Config.getString("password", "");
					if(mail.equals("")) {
						Logger.warning("nlAutoLogin: `mail' is not set!");
					}
					if(password.equals("")) {
						Logger.warning("nlAutoLogin: `password' is not set!");
					}
					String postStr = "mail=" + mail + "&password=" + password;
					PrintStream ps = new PrintStream(os);
					ps.print(postStr);
					ps.close();

					// Check if authentification succeeds
					if(connection.getContentLength() == 0 &&
						connection.getHeaderField("Location") != null &&
						connection.getHeaderField(null).equals("HTTP/1.1 302 Found") &&
						connection.getHeaderField("Set-Cookie") != null) {

						// Set cookie and reload by META tag
						Logger.info("nlAutoLogin: Authentification succeeds, try to reload " + match.group());
						responseHeader.setMessageHeader("Set-Cookie", connection.getHeaderField("Set-Cookie"));
						content = "<html><meta http-equiv=\"refresh\" content=\"0;URL=" + match.group() + "\"/></html>";

					} else {
						Logger.warning("nlAutoLogin: Authentification is failed!");
					}
				}
			} catch (URISyntaxException e) {
				Logger.warning("nlAutoLogin: URISyntaxException is caught!");
				Logger.debug(e);
			} catch (MalformedURLException e) {
				Logger.warning("nlAutoLogin: MalformedURLException is caught!");
				Logger.debug(e);
			}
		} else {
			// Do nothing
		}
		return content;
	}
}
